# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import re

from azure.cli.core.util import b64_to_hex

import knack.events as events


def register_global_transforms(cli_ctx):
    cli_ctx.register_event(events.EVENT_INVOKER_TRANSFORM_RESULT, _resource_group_transform)
    cli_ctx.register_event(events.EVENT_INVOKER_TRANSFORM_RESULT, _x509_from_base64_to_hex_transform)


def _parse_id(strid):
    parsed = {}
    parts = re.split('/', strid)
    if parts[3].lower() != 'resourcegroups':
        raise KeyError()

    parsed['resource-group'] = parts[4]
    parsed['name'] = parts[8]
    return parsed


def _add_resource_group(obj):
    if isinstance(obj, list):
        for array_item in obj:
            _add_resource_group(array_item)
    elif isinstance(obj, dict):
        try:
            if 'resourcegroup' not in [x.lower() for x in obj.keys()]:
                if obj['id']:
                    obj['resourceGroup'] = _parse_id(obj['id'])['resource-group']
        except (KeyError, IndexError, TypeError):
            pass
        for item_key in obj:
            if item_key != 'sourceVault':
                _add_resource_group(obj[item_key])


def _add_x509_hex(obj):
    if isinstance(obj, list):
        for array_item in obj:
            _add_x509_hex(array_item)
    elif isinstance(obj, dict):
        try:
            if 'x509ThumbprintHex' not in obj:
                if obj['x509Thumbprint']:
                    obj['x509ThumbprintHex'] = b64_to_hex(obj['x509Thumbprint'])
        except (KeyError, IndexError, TypeError):
            pass
        for item_key in obj:
            _add_x509_hex(obj[item_key])


def _resource_group_transform(_, **kwargs):
    _add_resource_group(kwargs['event_data']['result'])


def _x509_from_base64_to_hex_transform(_, **kwargs):
    _add_x509_hex(kwargs['event_data']['result'])


def gen_dict_to_list_transform(key='value'):

    def _dict_to_list_transform(result):
        if hasattr(result, key):
            return getattr(result, key)
        return result

    return _dict_to_list_transform



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import sys
from knack.util import CLIError
from azure.cli.core.util import sdk_no_wait

from ._client_factory import network_client_factory


def list_network_resource_property(resource, prop):
    """ Factory method for creating list functions. """

    def list_func(cmd, resource_group_name, resource_name):
        client = getattr(network_client_factory(cmd.cli_ctx), resource)
        return client.get(resource_group_name, resource_name).__getattribute__(prop)

    func_name = 'list_network_resource_property_{}_{}'.format(resource, prop)
    setattr(sys.modules[__name__], func_name, list_func)
    return func_name


def get_network_resource_property_entry(resource, prop):
    """ Factory method for creating get functions. """

    def get_func(cmd, resource_group_name, resource_name, item_name):
        client = getattr(network_client_factory(cmd.cli_ctx), resource)
        parent = getattr(client.get(resource_group_name, resource_name), prop)
        if parent is None:
            parent = []
        result = next((x for x in parent if x.name.lower() == item_name.lower()), None)
        if not result:
            raise CLIError("Item '{}' does not exist on {} '{}'".format(
                item_name, resource, resource_name))
        return result

    func_name = 'get_network_resource_property_entry_{}_{}'.format(resource, prop)
    setattr(sys.modules[__name__], func_name, get_func)
    return func_name


def delete_network_resource_property_entry(resource, prop):
    """ Factory method for creating delete functions. """

    def delete_func(cmd, resource_group_name, resource_name, item_name, no_wait=False):  # pylint: disable=unused-argument
        client = getattr(network_client_factory(cmd.cli_ctx), resource)
        item = client.get(resource_group_name, resource_name)

        if item.__getattribute__(prop) is not None:
            keep_items = [x for x in item.__getattribute__(prop) if x.name.lower() != item_name.lower()]
        else:
            keep_items = None

        with cmd.update_context(item) as c:
            c.set_param(prop, keep_items)
        if no_wait:
            sdk_no_wait(no_wait, client.begin_create_or_update, resource_group_name, resource_name, item)
        else:
            result = sdk_no_wait(no_wait, client.begin_create_or_update,
                                 resource_group_name, resource_name, item).result()
            if next((x for x in getattr(result, prop) or [] if x.name.lower() == item_name.lower()), None):
                raise CLIError("Failed to delete '{}' on '{}'".format(item_name, resource_name))

    func_name = 'delete_network_resource_property_entry_{}_{}'.format(resource, prop)
    setattr(sys.modules[__name__], func_name, delete_func)
    return func_name
